//**************************************************************************************************
//		C program for Runge-Kutta calculation in work station
//
//		macro version
//**************************************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#define MAXPATH 256

// file header size
#define LEN_HEADER 160
#define PTR_VERSION 144
#define LEN_PATH 256
#define LEN_STRING 512
#define LEN_TAB 16

// symbol status(attribute1)
#define SYM_CONSTANT 0x1
#define SYM_VARIABLE 0x2
#define SYM_INTEGRAL 0x4
#define SYM_GLOBAL 0x10
#define SYM_SOLUBLE 0x80
#define SYM_MEMBRANE 0xc100
#define SYM_DIFFUSION 0x200
#define SYM_CIRCUIT 0x400
#define SYM_REACTION 0x800
#define SYM_EQUATION 0x1000
#define SYM_STIMULATION 0x2000
#define SYM_TRANS 0x100
#define SYM_EXTRA 0x4000
#define SYM_INTRA 0x8000

// symbol attribute2
// symbol attribute3
// symbol attribute4

// axis
#define AXIS 6
#define PX 0
#define MX 1
#define PY 2
#define MY 3
#define PZ 4
#define MZ 5

// membrane adjacence
#define MEM_PX 0x01
#define MEM_MX 0x02
#define MEM_PY 0x04
#define MEM_MY 0x08
#define MEM_PZ 0x10
#define MEM_MZ 0x20

#define MEM_ADJ 8
#define X_PY	0
#define X_MY	1
#define X_PZ	2
#define X_MZ	3
#define X_PVY	4
#define X_MVY	5
#define X_PVZ	6
#define X_MVZ	7

#define Y_PX	0
#define Y_MX	1
#define Y_PZ	2
#define Y_MZ	3
#define Y_PVX	4
#define Y_MVX	5
#define Y_PVZ	6
#define Y_MVZ	7

#define Z_PX	0
#define Z_MX	1
#define Z_PY	2
#define Z_MY	3
#define Z_PVX	4
#define Z_MVX	5
#define Z_PVY	6
#define Z_MVY	7





// Shape Status
#define N_COMPARTMENT	2085
#define N_MEMBRANE	0
#define N_SYMBOL	5
#define N_SOLUBLE	5
#define N_TRANS	0
#define N_EXTRA	0
#define N_INTRA	0
#define N_STIMULATION	1
#define N_GLOBAL	0

#define MAX_SEPARATE_MEM	0
#define N_VARIABLE_SOLUBLE	3
#define N_VARIABLE_TRANS	0
#define N_VARIABLE_EXTRA	0
#define N_VARIABLE_INTRA	0

#define N_DIFFUSION_SOLUBLE	1
#define N_DIFFUSION_TRANS	0
#define N_DIFFUSION_EXTRA	0
#define N_DIFFUSION_INTRA	0

#define N_GLOBAL_SOLUBLE	0
#define N_GLOBAL_TRANS	0
#define N_GLOBAL_EXTRA	0
#define N_GLOBAL_INTRA	0

#define N_REACTION	2

// Compartment Scale [m]
#define LX	9.704248639352266E-7
#define LY	9.704248639352266E-7
#define LZ	1.0E-6
// Scale^2
#define LXLX	9.41724416543703E-13
#define LYLY	9.41724416543703E-13
#define LZLZ	1.0E-12
#define LXYLX	9.41724416543703E-13 //(LX+LY)*LX/2.0
#define LXYLY	9.41724416543703E-13
#define LYZLY	9.56074640239465E-13
#define LYZLZ	9.852124319676133E-13
#define LXZLZ	9.852124319676133E-13
#define LXZLX	9.56074640239465E-13
// membrane area[m^2]
#define SXY	9.41724416543703E-13
#define SXZ	9.704248639352266E-13
#define SYZ	9.704248639352266E-13


// Membrane conversion coefficient [m/molecule] : AA = 1/(6.022045e+26*Lx*Ly*Lz)
#define AA	1.7633242201098225E-9
#define _A	5.671106814024924E8 // The reciprocal of AA



// compartment 
typedef struct tab_compartment COMPARTMENT;
struct tab_compartment{
	//serial number of compartment
	long sncpm;
	//serial number of membrane
	long snmem[AXIS];

	//coordinates
	long x;
	long y;
	long z;

	//reactions
	unsigned char reaction[N_REACTION];

	//pointer to soluble symbol array.
	double* pSol;

	//last value of soluble symbols of next compartments
#if N_DIFFUSION_SOLUBLE
	double* pLastSol[AXIS];
#endif

	//last value of soluble symbols of next membranes
#if N_DIFFUSION_TRANS
	double* pLastTrans[AXIS][MEM_ADJ];
#endif
#if N_DIFFUSION_EXTRA
	double* pLastExtra[AXIS][MEM_ADJ];
#endif
#if N_DIFFUSION_INTRA
	double* pLastIntra[AXIS][MEM_ADJ];
#endif

	//membrane flag
	unsigned char memflag;

	//next 6 compartments (px,mx,py,my,pz,mz)
	COMPARTMENT* cpmAdj[AXIS];

	//serial number of next membranes
	long memAdj[AXIS][MEM_ADJ];

	//values of membrane symbol [transfer]
	double* pTrans[AXIS];
	//values of membrane symbol [extra]
	double* pExtra[AXIS];
	//values of membrane symbol [intra]
	double* pIntra[AXIS];

	//adjacent membranes for equivalent circuit
	long	cirAdj[AXIS][MEM_ADJ];

	//membrane area
	double	MA;

	//organella index
	long organella_index;

	//area density
	double area_density;
};





// define structure for the symbols, values and units
typedef struct tag_symbol_data{
	long	status;
	long	attribute2;
	long	attribute3;
	long	attribute4;
	double	value;
	double	diff_const;

	double	globularLength;	//Length of G-actin (default:10nm)
	double	transSpeed;	//transfer speed
	long	transDir;		//transfer direction
	double	adhesion;		//adhesion strength
	double	hankAngle;	//hank angle[rad](default:30 degree)
	double	ramusAngle;	//ramus angle[rad](default:70 degree)

	char	name[MAXPATH];	//only for simulation program
	char	unit[MAXPATH];	//A-Cell-3D:infinit

	long	order;
}SYMBOL;

// define structure for the stimulation condition
typedef struct tag_stimulation_data{
	int fnc;
	double mn;
	double mx;
	double amp;
	double stt;
	double stp;
	double frq;
	double t1;
	double t2;
	double BL;
	double BI;
	double dty;
	double pw;
	double t_on;
	double t_off;

	double ts0;
	double ts1;
	double tb0;
	double tb1;
	int burst;
}STIMULATION;

// define structure for the soluble diffusion constant
typedef struct tag_soluble_diffusion_constant{
	int		type;
	double	x;
	double	y;
	double	z;
}SOLDIFFUSION;

// define structure for the membrane diffusion constant
typedef struct tag_membrane_diffusion_constant{
	double	x_y;
	double	x_z;
	double	y_z;
	double	y_x;
	double	z_x;
	double	z_y;
	double	x_yc;
	double	x_zc;
	double	y_zc;
	double	y_xc;
	double	z_xc;
	double	z_yc;
}MEMDIFFUSION;

#ifdef LONG_MAX
#undef LONG_MAX
#endif
#ifdef LONG_MIN
#undef LONG_MIN
#endif

#define LONG_MAX 2147483647
#define LONG_MIN -2147483647-1

class CCell
{
public:
	int cntCpm;
	COMPARTMENT**** map;
	int xs,ys,zs;
	int maxX,maxY,maxZ;
	int minX,minY,minZ;
	int offsetX,offsetY,offsetZ;

	CCell()
	{
		cntCpm = 0;
		map = NULL;
		xs = ys = zs = 0;
		maxX = maxY = maxZ = LONG_MIN;
		minX = minY = minZ = LONG_MAX;
	}

	~CCell()
	{
		Free();
	}

	void Free()
	{
		if( map )
		{
			for( int x=0;x<xs;x++ )
			{
				for( int y=0;y<ys;y++ )
				{
					delete map[x][y];
				}
				delete map[x];
			}
			delete map;
			map = NULL;
		}
	}

	void SearchMembraneAdjacence( COMPARTMENT**** map, long x, long y, long z, unsigned char targetflag, long searchdir )
	{
		COMPARTMENT* p;
		long dir;
		long hadj,vadj;
		unsigned char sflag, vflag;
		long hx=x,hy=y,hz=z;
		long vx=x,vy=y,vz=z;
		long sdir,vdir;
		switch( targetflag )
		{
		case MEM_PX:
		case MEM_MX:
			if( targetflag==MEM_PX ){ dir=PX; vx+=1; }
			if( targetflag==MEM_MX ){ dir=MX; vx-=1; }
			switch( searchdir ){
			case PY: sflag=MEM_PY; vflag=MEM_MY; hadj=X_PY; vadj=X_PVY; hy+=1; vy+=1; sdir=PY; vdir=MY; break;
			case MY: sflag=MEM_MY; vflag=MEM_PY; hadj=X_MY; vadj=X_MVY; hy-=1; vy-=1; sdir=MY; vdir=PY; break;
			case PZ: sflag=MEM_PZ; vflag=MEM_MZ; hadj=X_PZ; vadj=X_PVZ; hz+=1; vz+=1; sdir=PZ; vdir=MZ; break;
			case MZ: sflag=MEM_MZ; vflag=MEM_PZ; hadj=X_MZ; vadj=X_MVZ; hz-=1; vz-=1; sdir=MZ; vdir=PZ; break;
			}
			break;
		case MEM_PY:
		case MEM_MY:
			if( targetflag==MEM_PY ){ dir=PY; vy+=1; }
			if( targetflag==MEM_MY ){ dir=MY; vy-=1; }
			switch( searchdir ){
			case PX: sflag=MEM_PX; vflag=MEM_MX; hadj=Y_PX; vadj=Y_PVX; hx+=1; vx+=1; sdir=PX; vdir=MX; break;
			case MX: sflag=MEM_MX; vflag=MEM_PX; hadj=Y_MX; vadj=Y_MVX; hx-=1; vx-=1; sdir=MX; vdir=PX; break;
			case PZ: sflag=MEM_PZ; vflag=MEM_MZ; hadj=Y_PZ; vadj=Y_PVZ; hz+=1; vz+=1; sdir=PZ; vdir=MZ; break;
			case MZ: sflag=MEM_MZ; vflag=MEM_PZ; hadj=Y_MZ; vadj=Y_MVZ; hz-=1; vz-=1; sdir=MZ; vdir=PZ; break;
			}
			break;
		case MEM_PZ:
		case MEM_MZ:
			if( targetflag==MEM_PZ ){ dir=PZ; vz+=1; }
			if( targetflag==MEM_MZ ){ dir=MZ; vz-=1; }
			switch( searchdir ){
			case PX: sflag=MEM_PX; vflag=MEM_MX; hadj=Z_PX; vadj=Z_PVX; hx+=1; vx+=1; sdir=PX; vdir=MX; break;
			case MX: sflag=MEM_MX; vflag=MEM_PX; hadj=Z_MX; vadj=Z_MVX; hx-=1; vx-=1; sdir=MX; vdir=PX; break;
			case PY: sflag=MEM_PY; vflag=MEM_MY; hadj=Z_PY; vadj=Z_PVY; hy+=1; vy+=1; sdir=PY; vdir=MY; break;
			case MY: sflag=MEM_MY; vflag=MEM_PY; hadj=Z_MY; vadj=Z_MVY; hy-=1; vy-=1; sdir=MY; vdir=PY; break;
			}
			break;
		}

		p = map[x][y][z];
		if( p->memflag&sflag ){
			p->memAdj[dir][vadj] = p->snmem[sdir];
			return;
		}
		if( map[hx][hy][hz] ){
			if( map[hx][hy][hz]->memflag&targetflag ){
				p->memAdj[dir][hadj] = map[hx][hy][hz]->snmem[dir];
				return;
			}
		}
		if( map[vx][vy][vz] ){
			if( map[vx][vy][vz]->memflag&vflag ){
				p->memAdj[dir][vadj] = map[vx][vy][vz]->snmem[vdir];
				return;
			}
		}

	}

	void SearchCircuitAdjacence( COMPARTMENT**** map, long x, long y, long z, unsigned char targetflag, long searchdir )
	{
		COMPARTMENT* p;
		long dir;
		long hadj,vadj;
		unsigned char sflag, vflag;
		long hx=x,hy=y,hz=z;
		long vx=x,vy=y,vz=z;
		long sdir,vdir;
		switch( targetflag )
		{
		case MEM_PX:
		case MEM_MX:
			if( targetflag==MEM_PX ){ dir=PX; vx+=1; }
			if( targetflag==MEM_MX ){ dir=MX; vx-=1; }
			switch( searchdir ){
			case PY: sflag=MEM_PY; vflag=MEM_MY; hadj=X_PY; vadj=X_PVY; hy+=1; vy+=1; sdir=PY; vdir=MY; break;
			case MY: sflag=MEM_MY; vflag=MEM_PY; hadj=X_MY; vadj=X_MVY; hy-=1; vy-=1; sdir=MY; vdir=PY; break;
			case PZ: sflag=MEM_PZ; vflag=MEM_MZ; hadj=X_PZ; vadj=X_PVZ; hz+=1; vz+=1; sdir=PZ; vdir=MZ; break;
			case MZ: sflag=MEM_MZ; vflag=MEM_PZ; hadj=X_MZ; vadj=X_MVZ; hz-=1; vz-=1; sdir=MZ; vdir=PZ; break;
			}
			break;
		case MEM_PY:
		case MEM_MY:
			if( targetflag==MEM_PY ){ dir=PY; vy+=1; }
			if( targetflag==MEM_MY ){ dir=MY; vy-=1; }
			switch( searchdir ){
			case PX: sflag=MEM_PX; vflag=MEM_MX; hadj=Y_PX; vadj=Y_PVX; hx+=1; vx+=1; sdir=PX; vdir=MX; break;
			case MX: sflag=MEM_MX; vflag=MEM_PX; hadj=Y_MX; vadj=Y_MVX; hx-=1; vx-=1; sdir=MX; vdir=PX; break;
			case PZ: sflag=MEM_PZ; vflag=MEM_MZ; hadj=Y_PZ; vadj=Y_PVZ; hz+=1; vz+=1; sdir=PZ; vdir=MZ; break;
			case MZ: sflag=MEM_MZ; vflag=MEM_PZ; hadj=Y_MZ; vadj=Y_MVZ; hz-=1; vz-=1; sdir=MZ; vdir=PZ; break;
			}
			break;
		case MEM_PZ:
		case MEM_MZ:
			if( targetflag==MEM_PZ ){ dir=PZ; vz+=1; }
			if( targetflag==MEM_MZ ){ dir=MZ; vz-=1; }
			switch( searchdir ){
			case PX: sflag=MEM_PX; vflag=MEM_MX; hadj=Z_PX; vadj=Z_PVX; hx+=1; vx+=1; sdir=PX; vdir=MX; break;
			case MX: sflag=MEM_MX; vflag=MEM_PX; hadj=Z_MX; vadj=Z_MVX; hx-=1; vx-=1; sdir=MX; vdir=PX; break;
			case PY: sflag=MEM_PY; vflag=MEM_MY; hadj=Z_PY; vadj=Z_PVY; hy+=1; vy+=1; sdir=PY; vdir=MY; break;
			case MY: sflag=MEM_MY; vflag=MEM_PY; hadj=Z_MY; vadj=Z_MVY; hy-=1; vy-=1; sdir=MY; vdir=PY; break;
			}
			break;
		}

		p = map[x][y][z];
		if( map[hx][hy][hz] ){
			if( map[hx][hy][hz]->memflag&targetflag ){
				p->cirAdj[dir][hadj] = map[hx][hy][hz]->sncpm;
				return;
			}
		}
		if( map[vx][vy][vz] ){
			if( map[vx][vy][vz]->memflag&vflag ){
				p->cirAdj[dir][vadj] = map[vx][vy][vz]->sncpm;
				return;
			}
		}
	}

	//read condition file and initialize.
	void Connect(COMPARTMENT* compartment, int cntCpm)
	{
		//get mapping size
		int i;
		int maxX,maxY,maxZ;
		int minX,minY,minZ;
		maxX = maxY = maxZ = LONG_MIN;
		minX = minY = minZ = LONG_MAX;
		for( i=0;i<cntCpm;i++ )
		{
			if( compartment[i].x < minX ) minX = compartment[i].x;
			if( compartment[i].x > maxX ) maxX = compartment[i].x;
			if( compartment[i].y < minY ) minY = compartment[i].y;
			if( compartment[i].y > maxY ) maxY = compartment[i].y;
			if( compartment[i].z < minZ ) minZ = compartment[i].z;
			if( compartment[i].z > maxZ ) maxZ = compartment[i].z;
		}
		xs = maxX-minX+1 + 2;
		ys = maxY-minY+1 + 2;
		zs = maxZ-minZ+1 + 2;
		offsetX = -minX+1;
		offsetY = -minY+1;
		offsetZ = -minZ+1;

		//create temporally file
		int x,y,z;
		map = (COMPARTMENT****)new void***[xs];
		for( x=0;x<xs;x++ )
		{
			map[x] = (COMPARTMENT***)new void**[ys];
			for( y=0;y<ys;y++ )
			{
				map[x][y] = (COMPARTMENT**)new void*[zs];
				for( z=0;z<zs;z++ )
				{
					map[x][y][z] = NULL;
				}
			}
		}
		for( i=0;i<cntCpm;i++ )
		{
			map[compartment[i].x+offsetX][compartment[i].y+offsetY][compartment[i].z+offsetZ] = &compartment[i];
		}

		// configuration for next compartment pointers.
		long xp,yp,zp;
		unsigned char mem_flag,aflag;
		for( x=0;x<xs;x++ )
		for( y=0;y<ys;y++ )
		for( z=0;z<zs;z++ )
		{
			if( map[x][y][z] )
			{
				for( i=0;i<AXIS;i++ )
				{
					switch (i)
					{
						case PX: xp=x+1;yp=y;zp=z;mem_flag=MEM_PX;aflag=MEM_MX;break;
						case MX: xp=x-1;yp=y;zp=z;mem_flag=MEM_MX;aflag=MEM_PX;break;
						case PY: xp=x;yp=y+1;zp=z;mem_flag=MEM_PY;aflag=MEM_MY;break;
						case MY: xp=x;yp=y-1;zp=z;mem_flag=MEM_MY;aflag=MEM_PY;break;
						case PZ: xp=x;yp=y;zp=z+1;mem_flag=MEM_PZ;aflag=MEM_MZ;break;
						case MZ: xp=x;yp=y;zp=z-1;mem_flag=MEM_MZ;aflag=MEM_PZ;break;
					}

					if( map[x][y][z]->memflag & mem_flag )//Is membrane
					{
					}else//Is not membrane
					if( map[xp][yp][zp] )//exists compartment
					{
						if( !(map[xp][yp][zp]->memflag & aflag) )//no membranes between compartments
						{
							//configure adjacent compartments
							map[x][y][z]->cpmAdj[i] = map[xp][yp][zp];
						}
					}
				}
			}
		}


		COMPARTMENT* pcpm;
		//configure adjacent membrane index
		for( x=0;x<xs;x++ )
		for( y=0;y<ys;y++ )
		for( z=0;z<zs;z++ )
		{
			if( map[x][y][z] )
			{
				pcpm = map[x][y][z];

				if( pcpm->memflag&MEM_PX )
				{
					SearchMembraneAdjacence( map,		//map
											x, y, z,	//location of target compartment
											MEM_PX,		//target membrane
											PY );		//searching orientation
					SearchMembraneAdjacence( map, x,y,z, MEM_PX, MY );
					SearchMembraneAdjacence( map, x,y,z, MEM_PX, PZ );
					SearchMembraneAdjacence( map, x,y,z, MEM_PX, MZ );
				}
				if( pcpm->memflag & MEM_MX )//+X is membrane
				{
					SearchMembraneAdjacence( map, x,y,z, MEM_MX, PY );
					SearchMembraneAdjacence( map, x,y,z, MEM_MX, MY );
					SearchMembraneAdjacence( map, x,y,z, MEM_MX, PZ );
					SearchMembraneAdjacence( map, x,y,z, MEM_MX, MZ );
				}
				if( pcpm->memflag&MEM_PY )
				{
					SearchMembraneAdjacence( map, x,y,z, MEM_PY, PX );
					SearchMembraneAdjacence( map, x,y,z, MEM_PY, MX );
					SearchMembraneAdjacence( map, x,y,z, MEM_PY, PZ );
					SearchMembraneAdjacence( map, x,y,z, MEM_PY, MZ );
				}
				if( pcpm->memflag&MEM_MY )
				{
					SearchMembraneAdjacence( map, x,y,z, MEM_MY, PX );
					SearchMembraneAdjacence( map, x,y,z, MEM_MY, MX );
					SearchMembraneAdjacence( map, x,y,z, MEM_MY, PZ );
					SearchMembraneAdjacence( map, x,y,z, MEM_MY, MZ );
				}
				if( pcpm->memflag&MEM_PZ )
				{
					SearchMembraneAdjacence( map, x,y,z, MEM_PZ, PX );
					SearchMembraneAdjacence( map, x,y,z, MEM_PZ, MX );
					SearchMembraneAdjacence( map, x,y,z, MEM_PZ, PY );
					SearchMembraneAdjacence( map, x,y,z, MEM_PZ, MY );
				}
				if( pcpm->memflag&MEM_MZ )
				{
					SearchMembraneAdjacence( map, x,y,z, MEM_MZ, PX );
					SearchMembraneAdjacence( map, x,y,z, MEM_MZ, MX );
					SearchMembraneAdjacence( map, x,y,z, MEM_MZ, PY );
					SearchMembraneAdjacence( map, x,y,z, MEM_MZ, MY );
				}
			}
		}


		//configure adjacent compartment with membrane(for Vm calculation)
		for( x=0;x<xs;x++ )
		for( y=0;y<ys;y++ )
		for( z=0;z<zs;z++ )
		{
			if( map[x][y][z]==NULL ) continue;

			pcpm = map[x][y][z];

			if( pcpm->memflag&MEM_PX )
			{
				SearchCircuitAdjacence( map,		//map
										x, y, z,	//location of target compartment
										MEM_PX,		//target membrane
										PY );		//searching orientation
				SearchCircuitAdjacence( map, x,y,z, MEM_PX, MY );
				SearchCircuitAdjacence( map, x,y,z, MEM_PX, PZ );
				SearchCircuitAdjacence( map, x,y,z, MEM_PX, MZ );
			}
			if( pcpm->memflag & MEM_MX )//+X$B$,KlLL(B
			{
				SearchCircuitAdjacence( map, x,y,z, MEM_MX, PY );
				SearchCircuitAdjacence( map, x,y,z, MEM_MX, MY );
				SearchCircuitAdjacence( map, x,y,z, MEM_MX, PZ );
				SearchCircuitAdjacence( map, x,y,z, MEM_MX, MZ );
			}
			if( pcpm->memflag&MEM_PY )
			{
				SearchCircuitAdjacence( map, x,y,z, MEM_PY, PX );
				SearchCircuitAdjacence( map, x,y,z, MEM_PY, MX );
				SearchCircuitAdjacence( map, x,y,z, MEM_PY, PZ );
				SearchCircuitAdjacence( map, x,y,z, MEM_PY, MZ );
			}
			if( pcpm->memflag&MEM_MY )
			{
				SearchCircuitAdjacence( map, x,y,z, MEM_MY, PX );
				SearchCircuitAdjacence( map, x,y,z, MEM_MY, MX );
				SearchCircuitAdjacence( map, x,y,z, MEM_MY, PZ );
				SearchCircuitAdjacence( map, x,y,z, MEM_MY, MZ );
			}
			if( pcpm->memflag&MEM_PZ )
			{
				SearchCircuitAdjacence( map, x,y,z, MEM_PZ, PX );
				SearchCircuitAdjacence( map, x,y,z, MEM_PZ, MX );
				SearchCircuitAdjacence( map, x,y,z, MEM_PZ, PY );
				SearchCircuitAdjacence( map, x,y,z, MEM_PZ, MY );
			}
			if( pcpm->memflag&MEM_MZ )
			{
				SearchCircuitAdjacence( map, x,y,z, MEM_MZ, PX );
				SearchCircuitAdjacence( map, x,y,z, MEM_MZ, MX );
				SearchCircuitAdjacence( map, x,y,z, MEM_MZ, PY );
				SearchCircuitAdjacence( map, x,y,z, MEM_MZ, MY );
			}
		}

	}


};



int InitSolubleSymbolArray(long* isl);
int InitTransMembraneSymbolArray(long* itr);
int InitExtraMembraneSymbolArray(long* iex);
int InitIntraMembraneSymbolArray(long* iin);
int InitSeparateMembraneSymbolArray(long* sepmem);

int InitVariableSolubleSymbolArray(long* vsl);
int InitVariableTransMembraneSymbolArray(long* vtr);
int InitVariableExtraMembraneSymbolArray(long* vex);
int InitVariableIntraMembraneSymbolArray(long* vin);

int InitDiffusibleSolubleSymbolArray(long* dsl);
int InitDiffusibleTransMembraneSymbolArray(long* dtr);
int InitDiffusibleExtraMembraneSymbolArray(long* dex);
int InitDiffusibleIntraMembraneSymbolArray(long* din);

int InitGlobalSolubleSymbolArray(long* refsl, long* gsl);
int InitGlobalTransMembraneSymbolArray(long* reftr, long* gtr);
int InitGlobalExtraMembraneSymbolArray(long* refex, long* gex);
int InitGlobalIntraMembraneSymbolArray(long* refin, long* gin);

//void Memset_long(long *tgtAry,int setprm,unsigned int setsize);
//void Memset_double(double *tgtAry,int setprm,unsigned int setsize);
void Memset_long(long *tgtAry,long setprm,unsigned int setsize);
void Memset_double(double *tgtAry,double setprm,unsigned int setsize);




int main(int argc, char **argv)
{

	// membrane area
	double S[AXIS]={SYZ,SYZ,SXZ,SXZ,SXY,SXY};

	// compartment array.
	static COMPARTMENT compartment[N_COMPARTMENT];

	//concentration of NULL symbols
	static double nullv[N_SYMBOL] = {0,};

	// soluble symbol array
#if N_SOLUBLE
	static double soluble[N_COMPARTMENT][N_SOLUBLE];
	static long	isl[N_SOLUBLE];
	InitSolubleSymbolArray(isl);
#endif

	// membrane symbol
#if N_TRANS
	static double trans[N_MEMBRANE][N_TRANS];
	static long itr[N_TRANS];
	InitTransMembraneSymbolArray(itr);
#endif
#if N_INTRA
	static double intra[N_MEMBRANE][N_INTRA];
	static long iin[N_INTRA];
	InitIntraMembraneSymbolArray(iin);
#endif
#if N_EXTRA
	static double extra[N_MEMBRANE][N_EXTRA];
	static long iex[N_EXTRA];
	InitExtraMembraneSymbolArray(iex);
#endif

	// pointers for soluble symbol array
	double* sl;			//soluble
	double* tr[AXIS];	//trans
	double* in[AXIS];	//intra
	double* ex[AXIS];	//extra



#ifdef CIRCUIT
	static double	pastVm[N_COMPARTMENT];
	static double	L_S[AXIS][MEM_ADJ];
#endif



	// static allocation ////////////////////////////////////
	static SYMBOL symbol[N_SYMBOL];


	// stimulation
#if N_STIMULATION
	static STIMULATION stimulation[N_STIMULATION];
	STIMULATION* stm;
	char* szname;
	double tbi;
#endif

	// integration
#if N_VARIABLE_SOLUBLE
	static double csl[N_VARIABLE_SOLUBLE];
	static double ksl[4][N_VARIABLE_SOLUBLE];
	static long vsl[N_VARIABLE_SOLUBLE];
	InitVariableSolubleSymbolArray(vsl);
#endif
#if N_VARIABLE_TRANS
	static double ctr[6][N_VARIABLE_TRANS];
	static double ktr[4][6][N_VARIABLE_TRANS];
	static long vtr[N_VARIABLE_TRANS];
	InitVariableTransMembraneSymbolArray(vtr);
#endif
#if N_VARIABLE_INTRA
	static double cin[6][N_VARIABLE_INTRA];
	static double kin[4][6][N_VARIABLE_INTRA];
	static long vin[N_VARIABLE_INTRA];
	InitVariableIntraMembraneSymbolArray(vin);
#endif
#if N_VARIABLE_EXTRA
	static double cex[6][N_VARIABLE_EXTRA];
	static double kex[4][6][N_VARIABLE_EXTRA];
	static long vex[N_VARIABLE_EXTRA];
	InitVariableExtraMembraneSymbolArray(vex);
#endif

	//global symbol
#if N_GLOBAL_SOLUBLE
	static long gsl[N_GLOBAL_SOLUBLE];
	static long refsl[N_GLOBAL_SOLUBLE];
	InitGlobalSolubleSymbolArray(refsl, gsl);
#endif
#if N_GLOBAL_TRANS
	static long gtr[N_GLOBAL_TRANS];
	static long reftr[N_GLOBAL_TRANS];
	InitGlobalTransMembraneSymbolArray(reftr, gtr);
#endif
#if N_GLOBAL_EXTRA
	static long gex[N_GLOBAL_EXTRA];
	static long refex[N_GLOBAL_EXTRA];
	InitGlobalExtraMembraneSymbolArray(refex, gex);
#endif
#if N_GLOBAL_INTRA
	static long gin[N_GLOBAL_INTRA];
	static long refin[N_GLOBAL_INTRA];
	InitGlobalIntraMembraneSymbolArray(refin, gin);
#endif


	// file, LEN_STRING, calculation data ///////////////////////
	int i, j, k, m;
	int x, y, z;
	int count;
	int len;

	// for reading condition files
	FILE *fp1, *fp2;
	char flc[MAXPATH],flo[MAXPATH];
	char w[LEN_STRING];
	char item[MAXPATH];
	unsigned char flag, flag2;
	char** szOutSym;	// output symbol name
	long adj=0;
	double lenCpmX,lenCpmY,lenCpmZ;	//size of compartment

	//variables for calculation
	COMPARTMENT* cpm;
	long numCpm=N_COMPARTMENT;
	long numMem=N_MEMBRANE;
	int n,nn,nm;
	long int e1,e2,nc,n1,n2,n4,n5;
	time_t t;
	unsigned char* reaction;
	double xstt,xstp,calstp,outstp,instp_h,instp_6,xx,tt;
	static int o_v[N_SYMBOL];
	int n_stm,n_sym,n_outval;
	double*	symval;
	double	value;

	long transform=0;

	//******************************** list of symbols and symbol number *********************************
	//*	sl[0] : B (M)
	//*	sl[1] : Ca^2^+ (M), diffusible
	//*	sl[2] : Ca^2^+B (M)
	//*	sl[3] : k_b (/s)
	//*	sl[4] : k_f (/M/s)
	//****************************************************************************************************


	// SIM.TMP file read and initial file write ****************************************************

	//read SIM.tmp
	if( (fp1=fopen( "SIM.tmp", "r" )) == NULL ){ printf("open error(SIM.tmp)\n"); exit(0); }
	fscanf( fp1, "%s", flc);	// flc:condition file name
	fclose( fp1 );

	

	//*********************************************************************************************
	//read condition file/create and write cal file 
	//
	if( (fp1=fopen( flc, "r" )) == NULL ){ printf("open error(flc)\n"); exit(0); }
	// read output file name
	fscanf( fp1, "%s", item);	// item="<OutputFile>"
	fscanf( fp1, "%s", flo);	// flo:output file name

	if( (fp2=fopen( flo, "wb" )) == NULL ){ printf("open error(flo)\n"); exit(0); }
	memset(w,'\0',LEN_HEADER);
	sprintf(&(w[PTR_VERSION]), "version=4.0");
	fwrite(w, sizeof(char), LEN_HEADER, fp2);

	// write information
	fscanf( fp1, "%s", item);	// item = "<Information>"
	fgets( w, LEN_STRING, fp1 );//read CRLF
	fgets( w, LEN_STRING, fp1 );
	len = strlen(w)-1;//delete CRLF
	w[len] = '\0';
	fwrite("*Information****", sizeof(char), LEN_TAB, fp2);
	fwrite(&len, sizeof(int), 1, fp2);
	fwrite(w, sizeof(char), len, fp2);

	// read Vsim file name
	fscanf( fp1, "%s", item);	// item="<VsimFile>"
	fgets( w, LEN_STRING, fp1 );//read CRLF
	fgets( w, LEN_STRING, fp1 );
	len = strlen(w)-1;//delete CRLF
	//memset(&w[len],'\0',LEN_HEADER-len);
	memset(&w[len],'\0',LEN_STRING-len);
	fwrite("*VsimFile*******", sizeof(char), LEN_TAB, fp2);
	fwrite(&len, sizeof(int), 1, fp2);
	fwrite(w, sizeof(char), LEN_STRING, fp2);


	// read/write symbols, units ...................................................................
	fscanf( fp1, "%s", item);	// item="<MembersOfSymbol>"
	fscanf( fp1, "%d", &n_sym );
	fwrite("*Symbol List****", sizeof(char), LEN_TAB, fp2);
	fwrite(&n_sym, sizeof(int), 1, fp2);
	for(i=0;i<n_sym;++i){
		fscanf( fp1, "%s %s %lx %lx %lx %lx %lf %lf %lf %lf %ld %lf %lf %lf",
						symbol[i].name,
						symbol[i].unit,
						&symbol[i].status,
						&symbol[i].attribute2,
						&symbol[i].attribute3,
						&symbol[i].attribute4,
						&symbol[i].value,
						&symbol[i].diff_const,
						&symbol[i].globularLength,
						&symbol[i].transSpeed,
						&symbol[i].transDir,
						&symbol[i].adhesion,
						&symbol[i].hankAngle,
						&symbol[i].ramusAngle );

		len = strlen( symbol[i].name );
		fwrite( &len, sizeof(int), 1, fp2 );
		fwrite(symbol[i].name, sizeof(char), len, fp2);

		len = strlen( symbol[i].unit );
		fwrite( &len, sizeof(int), 1, fp2 );
		fwrite(symbol[i].unit, sizeof(char), len, fp2);

		fwrite(&symbol[i].status, sizeof(int), 1, fp2);
		fwrite(&symbol[i].attribute2, sizeof(int), 1, fp2);
		fwrite(&symbol[i].attribute3, sizeof(int), 1, fp2);
		fwrite(&symbol[i].attribute4, sizeof(int), 1, fp2);
		fwrite(&symbol[i].value, sizeof(double), 1, fp2);
		fwrite(&symbol[i].diff_const, sizeof(double), 1, fp2);
		fwrite(&symbol[i].globularLength, sizeof(double), 1, fp2);
		fwrite(&symbol[i].transSpeed, sizeof(double), 1, fp2);
		fwrite(&symbol[i].transDir, sizeof(int), 1, fp2);
		fwrite(&symbol[i].adhesion, sizeof(double), 1, fp2);
		fwrite(&symbol[i].hankAngle, sizeof(double), 1, fp2);
		fwrite(&symbol[i].ramusAngle, sizeof(double), 1, fp2);
	}
	// set order
	int nsl = 0;
	int ntr = 0;
	int nex = 0;
	int nin = 0;
	for (i=0; i<n_sym; ++i)
	{
		if ((symbol[i].status&SYM_CIRCUIT) || !(symbol[i].status&SYM_MEMBRANE))
			symbol[i].order = nsl++;
		else if( symbol[i].status&SYM_TRANS )
			symbol[i].order = ntr++;
		else if( symbol[i].status&SYM_EXTRA )
			symbol[i].order = nex++;
		else if( symbol[i].status&SYM_INTRA )
			symbol[i].order = nin++;
	}

	// cal.start/cal.stop/cal.step/out.step ........................................................
	fscanf( fp1, "%s", item);	// item="<CalcCondition>"
	fscanf( fp1, "%s %lf", w, &xstt );
	fscanf( fp1, "%s %lf", w, &xstp );
	fscanf( fp1, "%s %lf", w, &calstp );
	fscanf( fp1, "%s %lf", w, &outstp );
	fwrite("*CalcCondition**", sizeof(char), LEN_TAB, fp2);
	fwrite(&xstt, sizeof(double), 1, fp2);
	fwrite(&xstp, sizeof(double), 1, fp2);
	fwrite(&calstp, sizeof(double), 1, fp2);
	fwrite(&outstp, sizeof(double), 1, fp2);

	// read number of outval .......................................................................
	fscanf( fp1, "%s", item);	// item="<OutputSymbols>"
	fscanf( fp1, "%d", &n_outval );
	fwrite("*OutputSymbols**", sizeof(char), LEN_TAB, fp2);
	fwrite(&n_outval, sizeof(int), 1, fp2);
	szOutSym = (char**)malloc(sizeof(char*)*n_outval);
	for(i=0; i<n_outval; ++i)
	{
		szOutSym[i] = (char*)malloc(MAXPATH);
		fscanf( fp1, "%s", szOutSym[i] );
		len = strlen( szOutSym[i] );
		fwrite(&len, sizeof(int), 1, fp2);
		fwrite(szOutSym[i], sizeof(char), len, fp2);
	}
	// set output symbol array
	for (i=0; i<n_outval; ++i) {
		for (j=0; j<n_sym; ++j) {
			if (strcmp(szOutSym[i], symbol[j].name) == 0) {o_v[i]=j; break;}
		}
		if (o_v[i] < 0) exit(0);
	}
	for (i=0; i<n_outval; ++i)
		free(szOutSym[i]);
	free(szOutSym);
	

	// read stimulation condition ..................................................................
	fscanf( fp1, "%s", item);	// item="<Stimulation>"
	fscanf( fp1, "%d", &n_stm );
	fwrite("*Stimulation****", sizeof(char), LEN_TAB, fp2 );
	fwrite(&n_stm, sizeof(int), 1, fp2);
#if N_STIMULATION
	szname = (char*)malloc(MAXPATH);
	for(n=0;n<n_stm;n++) {
		fscanf( fp1, "%s", szname );
		fscanf( fp1, "%s %d", w, &stimulation[n].fnc );
		fscanf( fp1, "%s %lf", w, &stimulation[n].mn );
		fscanf( fp1, "%s %lf", w, &stimulation[n].mx );
		fscanf( fp1, "%s %lf", w, &stimulation[n].amp );
		fscanf( fp1, "%s %lf", w, &stimulation[n].stt );
		fscanf( fp1, "%s %lf", w, &stimulation[n].stp );
		fscanf( fp1, "%s %lf", w, &stimulation[n].frq );
		fscanf( fp1, "%s %lf", w, &stimulation[n].t1 );
		fscanf( fp1, "%s %lf", w, &stimulation[n].t2 );
		fscanf( fp1, "%s %lf", w, &stimulation[n].BL );
		fscanf( fp1, "%s %lf", w, &stimulation[n].BI );
		fscanf( fp1, "%s %lf", w, &stimulation[n].dty );
		fscanf( fp1, "%s %lf", w, &stimulation[n].pw );
		fscanf( fp1, "%s %lf", w, &stimulation[n].t_on );
		fscanf( fp1, "%s %lf", w, &stimulation[n].t_off );

		len = strlen( szname );
		fwrite( &len, sizeof(int), 1, fp2 );
		fwrite( szname, sizeof(char), len, fp2 );
		fwrite( &stimulation[n].fnc, sizeof(int), 1, fp2 );
		fwrite( &stimulation[n].mn, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].mx, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].amp, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].stt, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].stp, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].frq, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].t1, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].t2, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].BL, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].BI, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].dty, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].pw, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].t_on, sizeof(double), 1, fp2 );
		fwrite( &stimulation[n].t_off, sizeof(double), 1, fp2 );
	}
	free(szname);
#endif


	// read shape information symbol/shape .........................................................
	fscanf( fp1, "%s", item);	// item="<CellStructure>"
	fgets( w, LEN_STRING, fp1 );//read CRLF
	fgets( w, LEN_STRING, fp1 );
	fscanf( fp1, "%ld %ld %lf %lf %lf", &numCpm, &numMem, &lenCpmX, &lenCpmY, &lenCpmZ );
	fwrite("*CompartmentSt.*", sizeof(char), LEN_TAB, fp2);
	fwrite(&numCpm, sizeof(int), 1, fp2);
	fwrite(&numMem, sizeof(int), 1, fp2);
	fwrite(&lenCpmX, sizeof(double), 1, fp2);
	fwrite(&lenCpmY, sizeof(double), 1, fp2);
	fwrite(&lenCpmZ, sizeof(double), 1, fp2);

	fgets( w, LEN_STRING, fp1 );//read CRLF
	fgets( w, LEN_STRING, fp1 );
	fgets( w, LEN_STRING, fp1 );
	long parts;
	long numStruct;
	count = 0;//membrane counter
	for (i=0; i<numCpm; ++i)
	{
		cpm = &compartment[i];

		//compartment index
		cpm->sncpm = i;
		//read compartment info.
		fscanf( fp1, "%ld %ld %ld %x %lx %ld %ld %lf",
						&cpm->x,
						&cpm->y,
						&cpm->z,
						&cpm->memflag,
						&parts,
						&numStruct,
						&cpm->organella_index,
						&cpm->area_density);
		//read reaction flag
		fgets( w, LEN_STRING, fp1 );//read CRLF
		for( j=0;j<N_REACTION;j++ )
		{
			fgets( w, sizeof(unsigned char)+1, fp1 );
			if( w[0]=='1' )
				cpm->reaction[j] = 1;
			else
				cpm->reaction[j] = 0;
		}

		fwrite(&(cpm->x), sizeof(int), 1, fp2);
		fwrite(&(cpm->y), sizeof(int), 1, fp2);
		fwrite(&(cpm->z), sizeof(int), 1, fp2);
		fwrite(&cpm->memflag, sizeof(unsigned char), 1, fp2);
		fwrite(&parts, sizeof(int), 1, fp2);
		fwrite(&numStruct, sizeof(int), 1, fp2);
		cpm->pSol = NULL;
		unsigned char mem_flag = 0x1;
		for( j=0;j<AXIS;j++ )
		{
			//membrane index
			if( cpm->memflag&mem_flag )
			{
				cpm->snmem[j] = count++;
			}else{
				cpm->snmem[j] = -1;
			}
			cpm->cpmAdj[j] = NULL;
			Memset_long( &(cpm->memAdj[j][0]) ,-1,MEM_ADJ);
			cpm->pTrans[j] = nullv;
			cpm->pExtra[j] = nullv;
			cpm->pIntra[j] = nullv;
#ifdef CIRCUIT
			Memset_long( &(cpm->cirAdj[j][0]) ,-1,MEM_ADJ);
#endif

			mem_flag<<=1;//move membrane flag
		}
		cpm->MA = 0;
#ifdef CIRCUIT
		if (cpm->memflag & MEM_PX) cpm->MA += SYZ;
		if (cpm->memflag & MEM_MX) cpm->MA += SYZ;
		if (cpm->memflag & MEM_PY) cpm->MA += SXZ;
		if (cpm->memflag & MEM_MY) cpm->MA += SXZ;
		if (cpm->memflag & MEM_PZ) cpm->MA += SXY;
		if (cpm->memflag & MEM_MZ) cpm->MA += SXY;
#endif
	}

	//read concentration
	fwrite("*SymbolValues***", sizeof(char), LEN_TAB, fp2);
	int nsol=0,ntra=0,next=0,nint=0;
	for( i=0;i<N_SYMBOL;i++ )
	{
#if N_SOLUBLE
		if( (symbol[i].status&SYM_CIRCUIT) && (symbol[i].status&SYM_MEMBRANE) )
		{
			fgets( w, LEN_STRING, fp1 );//read CRLF
			fgets( w, LEN_STRING, fp1 );//read comments
			for( j=0;j<N_COMPARTMENT;j++ )
			{
				soluble[j][nsol] = 0;
				for( m=0;m<AXIS;m++ )
				{
					if( compartment[j].snmem[m]>=0 )
					{
						fscanf( fp1, "%lf", &value );
						fwrite(&value, sizeof(double), 1, fp2);
						soluble[j][nsol] += value*S[m];
					}
				}
			}
			nsol++;
			continue;
		}
		if( symbol[i].status&SYM_SOLUBLE )
		{
			fgets( w, LEN_STRING, fp1 );//read CRLF
			fgets( w, LEN_STRING, fp1 );//read comments
			for( j=0;j<N_COMPARTMENT;j++ )
			{
				fscanf( fp1, "%lf", &soluble[j][nsol] );
				fwrite(&soluble[j][nsol], sizeof(double), 1, fp2);
			}
			nsol++;
			continue;
		}
#endif
#if N_TRANS
		if( symbol[i].status & SYM_TRANS )
		{
			fgets( w, LEN_STRING, fp1 );//read CRLF
			fgets( w, LEN_STRING, fp1 );//read comments
			for( j=0;j<N_MEMBRANE;j++ )
			{
				fscanf( fp1, "%lf", &trans[j][ntra] );
				fwrite(&trans[j][ntra], sizeof(double), 1, fp2);
			}
			ntra++;
			continue;
		}
#endif
#if N_EXTRA
		if( symbol[i].status & SYM_EXTRA )
		{
			fgets( w, LEN_STRING, fp1 );//read CRLF
			fgets( w, LEN_STRING, fp1 );//read comments
			for( j=0;j<N_MEMBRANE;j++ )
			{
				fscanf( fp1, "%lf", &extra[j][next] );
				fwrite(&extra[j][next], sizeof(double), 1, fp2);
			}
			next++;
			continue;
		}
#endif
#if N_INTRA
		if( symbol[i].status & SYM_TRANS )
		{
			fgets( w, LEN_STRING, fp1 );//read CRLF
			fgets( w, LEN_STRING, fp1 );//read comments
			for( j=0;j<N_MEMBRANE;j++ )
			{
				fscanf( fp1, "%lf", &intra[j][nint] );
				fwrite(&intra[j][nint], sizeof(double), 1, fp2);
			}
			nint++;
			continue;
		}
#endif
	}

	fclose(fp2);
	fclose(fp1);

	//*********************************************************************************************
	//Calculate molar number on membrane
	for (i=0; i<numCpm; ++i)
	{
		cpm = &compartment[i];
		for( j=0;j<AXIS;j++ )
		{
			if( cpm->snmem[j]>=0 )
			{
#if N_TRANS
				for(k=0;k<N_TRANS;k++) trans[cpm->snmem[j]][k]*=S[j];
#endif
#if N_INTRA
				for(k=0;k<N_INTRA;k++) intra[cpm->snmem[j]][k]*=S[j];
#endif
#if N_EXTRA
				for(k=0;k<N_EXTRA;k++) extra[cpm->snmem[j]][k]*=S[j];
#endif
			}
		}
	}


	//*********************************************************************************************
	//Connect compartments
	//

	//Set pointers for concentration array
	for (i=0; i<numCpm; ++i)
	{
		cpm = &compartment[i];
#if N_SOLUBLE
		cpm->pSol = soluble[i];
#endif
		for( j=0;j<AXIS;j++ )
		{
			if( cpm->snmem[j]>=0 )
			{
#if N_TRANS
				cpm->pTrans[j] = trans[cpm->snmem[j]];
#endif
#if N_INTRA
				cpm->pIntra[j] = intra[cpm->snmem[j]];
#endif
			}
		}
	}
	//Connect membrane to compartment
	CCell cell;
	cell.Connect(compartment, N_COMPARTMENT);
	long xp,yp,zp;
	for( x=0;x<cell.xs;x++ )
	for( y=0;y<cell.ys;y++ )
	for( z=0;z<cell.zs;z++ )
	{
		if( cell.map[x][y][z] )
		{
			unsigned char mem_flag = 0x1;
			for( i=0;i<AXIS;i++ )
			{
				if( cell.map[x][y][z]->snmem[i]>=0 )
				{
					switch (i)
					{
						case PX:xp=x+1;yp=y;zp=z;break;
						case MX:xp=x-1;yp=y;zp=z;break;
						case PY:xp=x;yp=y+1;zp=z;break;
						case MY:xp=x;yp=y-1;zp=z;break;
						case PZ:xp=x;yp=y;zp=z+1;break;
						case MZ:xp=x;yp=y;zp=z-1;break;
					}
					if( cell.map[xp][yp][zp] )
					{
#if N_TRANS
						cell.map[xp][yp][zp]->pTrans[MX] = trans[cell.map[x][y][z]->snmem[i]];
#endif
#if N_EXTRA
						cell.map[xp][yp][zp]->pExtra[MX] = extra[cell.map[x][y][z]->snmem[i]];
#endif
					}
				}
			}
		}
	}
	cell.Free();


	//*********************************************************************************************
	//Configuration for soluble diffusion
	//soluble diffusion
#if N_DIFFUSION_SOLUBLE
	static double CDS[N_DIFFUSION_SOLUBLE][AXIS];
	static double lastsol[N_COMPARTMENT][N_DIFFUSION_SOLUBLE];
	static long dsl[N_DIFFUSION_SOLUBLE];
	//Set soluble array lastsol[no_soluble_diffusion_symbol] = soluble[ids[no_soluble_diffusion_symbol]]
	InitDiffusibleSolubleSymbolArray(dsl);
	//Set constants
	for( i=0;i<N_DIFFUSION_SOLUBLE;i++ )
	{
		CDS[i][PX] = symbol[isl[dsl[i]]].diff_const / LXLX;
		CDS[i][MX] = symbol[isl[dsl[i]]].diff_const / LXLX;
		CDS[i][PY] = symbol[isl[dsl[i]]].diff_const / LYLY;
		CDS[i][MY] = symbol[isl[dsl[i]]].diff_const / LYLY;
		CDS[i][PZ] = symbol[isl[dsl[i]]].diff_const / LZLZ;
		CDS[i][MZ] = symbol[isl[dsl[i]]].diff_const / LZLZ;
	}
	//Set pointers from compartment to lastsol
	for( i=0;i<numCpm;i++ )
	{
		cpm = &compartment[i];
		for( j=0;j<AXIS;j++ )
		{
			if(cpm->cpmAdj[j])
			{
				cpm->pLastSol[j] = lastsol[cpm->cpmAdj[j]->sncpm];
			}else{
				cpm->pLastSol[j] = NULL;
			}
		}
	}
#endif

#if N_DIFFUSION_TRANS
	static double CDT[N_DIFFUSION_TRANS][AXIS][MEM_ADJ];
	static double lasttrans[N_MEMBRANE][N_DIFFUSION_TRANS];
	static long dtr[N_DIFFUSION_TRANS];
	InitDiffusibleTransMembraneSymbolArray(dtr);
	//Set constants
	for( i=0;i<N_DIFFUSION_TRANS;i++ )
	{
		for (m=0; m<2; ++m)	//CM[i][PX],CM[i][MX]
		{
			CDT[i][m][X_PY] = symbol[itr[dtr[i]]].diff_const / LYLY;
			CDT[i][m][X_PVY] = symbol[itr[dtr[i]]].diff_const / LXYLY;
			CDT[i][m][X_MY] = symbol[itr[dtr[i]]].diff_const / LYLY;
			CDT[i][m][X_MVY] = symbol[itr[dtr[i]]].diff_const / LXYLY;
			CDT[i][m][X_PZ] = symbol[itr[dtr[i]]].diff_const / LZLZ;
			CDT[i][m][X_PVZ] = symbol[itr[dtr[i]]].diff_const / LXZLZ;
			CDT[i][m][X_MZ] = symbol[itr[dtr[i]]].diff_const / LZLZ;
			CDT[i][m][X_MVZ] = symbol[itr[dtr[i]]].diff_const / LXZLZ;
		}
		for (m=2; m<4; ++m)	//CM[i][PY],CM[i][MY]
		{
			CDT[i][m][Y_PX] = symbol[itr[dtr[i]]].diff_const / LXLX;
			CDT[i][m][Y_PVX] = symbol[itr[dtr[i]]].diff_const / LXYLX;
			CDT[i][m][Y_MX] = symbol[itr[dtr[i]]].diff_const / LXLX;
			CDT[i][m][Y_MVX] = symbol[itr[dtr[i]]].diff_const / LXYLX;
			CDT[i][m][Y_PZ] = symbol[itr[dtr[i]]].diff_const / LZLZ;
			CDT[i][m][Y_PVZ] = symbol[itr[dtr[i]]].diff_const / LYZLZ;
			CDT[i][m][Y_MZ] = symbol[itr[dtr[i]]].diff_const / LZLZ;
			CDT[i][m][Y_MVZ] = symbol[itr[dtr[i]]].diff_const / LYZLZ;
		}
		for (m=4; m<6; ++m)	//CM[i][PZ],CM[i][MZ]
		{
			CDT[i][m][Z_PX] = symbol[itr[dtr[i]]].diff_const / LXLX;
			CDT[i][m][Z_PVX] = symbol[itr[dtr[i]]].diff_const / LXZLX;
			CDT[i][m][Z_MX] = symbol[itr[dtr[i]]].diff_const / LXLX;
			CDT[i][m][Z_MVX] = symbol[itr[dtr[i]]].diff_const / LXZLX;
			CDT[i][m][Z_PY] = symbol[itr[dtr[i]]].diff_const / LYLY;
			CDT[i][m][Z_PVY] = symbol[itr[dtr[i]]].diff_const / LYZLY;
			CDT[i][m][Z_MY] = symbol[itr[dtr[i]]].diff_const / LYLY;
			CDT[i][m][Z_MVY] = symbol[itr[dtr[i]]].diff_const / LYZLY;
		}
	}
	//Set pointers from compartment to lastsol
	for( i=0;i<numCpm;i++ )
	{
		cpm = &compartment[i];
		for( j=0;j<AXIS;j++ )
		for( k=0;k<MEM_ADJ;k++ )
		{
			if(cpm->memAdj[j][k]>-1)
			{
				cpm->pLastTrans[j][k] = lasttrans[cpm->memAdj[j][k]];
			}else{
				cpm->pLastTrans[j][k] = NULL;
			}
		}
	}
#endif

#if N_DIFFUSION_EXTRA
	static double CDE[N_DIFFUSION_EXTRA][AXIS][MEM_ADJ];
	static double lastextra[N_MEMBRANE][N_DIFFUSION_EXTRA];
	static long dex[N_DIFFUSION_EXTRA];
	InitDiffusibleExtraMembraneSymbolArray(dex);
	//Set constant
	for( i=0;i<N_DIFFUSION_EXTRA;i++ )
	{
		for (m=0; m<2; ++m)	//CM[i][PX],CM[i][MX]
		{
			CDE[i][m][X_PY] = symbol[itr[dex[i]]].diff_const / LYLY;
			CDE[i][m][X_PVY] = symbol[itr[dex[i]]].diff_const / LXYLY;
			CDE[i][m][X_MY] = symbol[itr[dex[i]]].diff_const / LYLY;
			CDE[i][m][X_MVY] = symbol[itr[dex[i]]].diff_const / LXYLY;
			CDE[i][m][X_PZ] = symbol[itr[dex[i]]].diff_const / LZLZ;
			CDE[i][m][X_PVZ] = symbol[itr[dex[i]]].diff_const / LXZLZ;
			CDE[i][m][X_MZ] = symbol[itr[dex[i]]].diff_const / LZLZ;
			CDE[i][m][X_MVZ] = symbol[itr[dex[i]]].diff_const / LXZLZ;
		}
		for (m=2; m<4; ++m)	//CM[i][PY],CM[i][MY]
		{
			CDE[i][m][Y_PX] = symbol[itr[dex[i]]].diff_const / LXLX;
			CDE[i][m][Y_PVX] = symbol[itr[dex[i]]].diff_const / LXYLX;
			CDE[i][m][Y_MX] = symbol[itr[dex[i]]].diff_const / LXLX;
			CDE[i][m][Y_MVX] = symbol[itr[dex[i]]].diff_const / LXYLX;
			CDE[i][m][Y_PZ] = symbol[itr[dex[i]]].diff_const / LZLZ;
			CDE[i][m][Y_PVZ] = symbol[itr[dex[i]]].diff_const / LYZLZ;
			CDE[i][m][Y_MZ] = symbol[itr[dex[i]]].diff_const / LZLZ;
			CDE[i][m][Y_MVZ] = symbol[itr[dex[i]]].diff_const / LYZLZ;
		}
		for (m=4; m<6; ++m)	//CM[i][PZ],CM[i][MZ]
		{
			CDE[i][m][Z_PX] = symbol[itr[dex[i]]].diff_const / LXLX;
			CDE[i][m][Z_PVX] = symbol[itr[dex[i]]].diff_const / LXZLX;
			CDE[i][m][Z_MX] = symbol[itr[dex[i]]].diff_const / LXLX;
			CDE[i][m][Z_MVX] = symbol[itr[dex[i]]].diff_const / LXZLX;
			CDE[i][m][Z_PY] = symbol[itr[dex[i]]].diff_const / LYLY;
			CDE[i][m][Z_PVY] = symbol[itr[dex[i]]].diff_const / LYZLY;
			CDE[i][m][Z_MY] = symbol[itr[dex[i]]].diff_const / LYLY;
			CDE[i][m][Z_MVY] = symbol[itr[dex[i]]].diff_const / LYZLY;
		}
	}
	//Set pointers from compartment to lastextra
	for( i=0;i<numCpm;i++ )
	{
		cpm = &compartment[i];
		for( j=0;j<AXIS;j++ )
		for( k=0;k<MEM_ADJ;k++ )
		{
			if(cpm->memAdj[j][k]>-1)
			{
				cpm->pLastExtra[j][k] = lastextra[cpm->memAdj[j][k]];
			}else{
				cpm->pLastExtra[j][k] = NULL;
			}
		}
	}
#endif

#if N_DIFFUSION_INTRA
	static double CDI[N_DIFFUSION_INTRA][AXIS][MEM_ADJ];
	static double lastintra[N_MEMBRANE][N_DIFFUSION_INTRA];
	static long din[N_DIFFUSION_INTRA];
	InitDiffusibleIntraMembraneSymbolArray(din);
	//Const$B%;%C%H(B
	for( i=0;i<N_DIFFUSION_INTRA;i++ )
	{
		for (m=0; m<2; ++m)	//CM[i][PX],CM[i][MX]
		{
			CDI[i][m][X_PY] = symbol[itr[din[i]]].diff_const / LYLY;
			CDI[i][m][X_PVY] = symbol[itr[din[i]]].diff_const / LXYLY;
			CDI[i][m][X_MY] = symbol[itr[din[i]]].diff_const / LYLY;
			CDI[i][m][X_MVY] = symbol[itr[din[i]]].diff_const / LXYLY;
			CDI[i][m][X_PZ] = symbol[itr[din[i]]].diff_const / LZLZ;
			CDI[i][m][X_PVZ] = symbol[itr[din[i]]].diff_const / LXZLZ;
			CDI[i][m][X_MZ] = symbol[itr[din[i]]].diff_const / LZLZ;
			CDI[i][m][X_MVZ] = symbol[itr[din[i]]].diff_const / LXZLZ;
		}
		for (m=2; m<4; ++m)	//CM[i][PY],CM[i][MY]
		{
			CDI[i][m][Y_PX] = symbol[itr[din[i]]].diff_const / LXLX;
			CDI[i][m][Y_PVX] = symbol[itr[din[i]]].diff_const / LXYLX;
			CDI[i][m][Y_MX] = symbol[itr[din[i]]].diff_const / LXLX;
			CDI[i][m][Y_MVX] = symbol[itr[din[i]]].diff_const / LXYLX;
			CDI[i][m][Y_PZ] = symbol[itr[din[i]]].diff_const / LZLZ;
			CDI[i][m][Y_PVZ] = symbol[itr[din[i]]].diff_const / LYZLZ;
			CDI[i][m][Y_MZ] = symbol[itr[din[i]]].diff_const / LZLZ;
			CDI[i][m][Y_MVZ] = symbol[itr[din[i]]].diff_const / LYZLZ;
		}
		for (m=4; m<6; ++m)	//CM[i][PZ],CM[i][MZ]
		{
			CDI[i][m][Z_PX] = symbol[itr[din[i]]].diff_const / LXLX;
			CDI[i][m][Z_PVX] = symbol[itr[din[i]]].diff_const / LXZLX;
			CDI[i][m][Z_MX] = symbol[itr[din[i]]].diff_const / LXLX;
			CDI[i][m][Z_MVX] = symbol[itr[din[i]]].diff_const / LXZLX;
			CDI[i][m][Z_PY] = symbol[itr[din[i]]].diff_const / LYLY;
			CDI[i][m][Z_PVY] = symbol[itr[din[i]]].diff_const / LYZLY;
			CDI[i][m][Z_MY] = symbol[itr[din[i]]].diff_const / LYLY;
			CDI[i][m][Z_MVY] = symbol[itr[din[i]]].diff_const / LYZLY;
		}
	}
	////Set pointers from compartment to lastintra
	for( i=0;i<numCpm;i++ )
	{
		cpm = &compartment[i];
		for( j=0;j<AXIS;j++ )
		for( k=0;k<MEM_ADJ;k++ )
		{
			if(cpm->memAdj[j][k]>-1)
			{
				cpm->pLastIntra[j][k] = lastintra[cpm->memAdj[j][k]];
			}else{
				cpm->pLastIntra[j][k] = NULL;
			}
		}
	}
#endif


#ifdef CIRCUIT
	for (m=0; m<2; ++m)	//L_S[PX],L_S[MX]
	{
		L_S[m][X_PY] = LY / SXZ;
		L_S[m][X_MY] = LY / SXZ;
		L_S[m][X_PZ] = LZ / SXY;
		L_S[m][X_MZ] = LZ / SXY;
		L_S[m][X_PVY] = LY / SXZ + LX/SYZ;
		L_S[m][X_MVY] = LY / SXZ + LX/SYZ;
		L_S[m][X_PVZ] = LY / SXZ + LX/SYZ;
		L_S[m][X_MVZ] = LY / SXZ + LX/SYZ;
	}
	for (m=2; m<4; ++m)	//L_S[PY],L_S[MY]
	{
		L_S[m][Y_PX] = LX / SYZ;
		L_S[m][Y_MX] = LX / SYZ;
		L_S[m][Y_PZ] = LZ / SXY;
		L_S[m][Y_MZ] = LZ / SXY;
		L_S[m][Y_PVX] = LX / SYZ + LY/SXZ;
		L_S[m][Y_MVX] = LX / SYZ + LY/SXZ;
		L_S[m][Y_PVZ] = LZ / SXY + LY/SXZ;
		L_S[m][Y_MVZ] = LZ / SXY + LY/SXZ;
	}
	for (m=4; m<6; ++m)	//L_S[PZ],L_S[MZ]
	{
		L_S[m][Z_PX] = LX / SYZ;
		L_S[m][Z_MX] = LX / SYZ;
		L_S[m][Z_PY] = LY / SXZ;
		L_S[m][Z_MY] = LY / SXZ;
		L_S[m][Z_PVX] = LX / SYZ + LZ/SXY;
		L_S[m][Z_MVX] = LX / SYZ + LZ/SXY;
		L_S[m][Z_PVY] = LY / SXZ + LZ/SXY;
		L_S[m][Z_MVY] = LY / SXZ + LZ/SXY;
	}
#endif




	//global symbol	
#if N_GLOBAL_SOLUBLE
			for (nc=0; nc<numCpm; ++nc)
			for (i=0; i<N_GLOBAL_SOLUBLE; ++i)
				soluble[nc][gsl[i]] = soluble[refsl[i]][gsl[i]];
#endif
#if N_GLOBAL_TRANS
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_GLOBAL_TRANS; ++i)
				trans[nc][gtr[i]] = trans[reftr[i]][gtr[i]];
#endif
#if N_GLOBAL_EXTRA
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_GLOBAL_EXTRA; ++i)
				extra[nc][gex[i]] = extra[refex[i]][gex[i]];
#endif
#if N_GLOBAL_INTRA
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_GLOBAL_INTRA; ++i)
				intra[nc][gin[i]] = intra[refin[i]][gin[i]];
#endif


	// save values .................................................................................
	if( (fp1=fopen(flo,"ab")) == NULL ){ printf("open error\n"); exit(0); }
	sprintf(w, "*results********");
	fwrite(w, sizeof(char), LEN_TAB, fp1);
	fwrite(&transform, sizeof(int), 1, fp1);
	for (i=0; i<n_outval; ++i)
	{
		for (nc=0; nc<numCpm; ++nc)
		{
			//if(compartment[nc]==NULL)
			//	continue;
			if ((symbol[o_v[i]].status&SYM_CIRCUIT) && (symbol[o_v[i]].status&SYM_MEMBRANE))
			{
				if (compartment[nc].MA != 0)
					value = compartment[nc].pSol[symbol[o_v[i]].order] / compartment[nc].MA;
				else
					value = 0;
				for( m=0;m<AXIS;m++ )
				{
					if( compartment[nc].snmem[m] >= 0 )
						fwrite(&value, sizeof(double), 1, fp1);
				}
			}else
			if ((symbol[o_v[i]].status&SYM_CIRCUIT) || !(symbol[o_v[i]].status&SYM_MEMBRANE))
			{
				fwrite(&(compartment[nc].pSol[symbol[o_v[i]].order]), sizeof(double), 1, fp1);
			}else
			if( symbol[o_v[i]].status&SYM_TRANS )
			{
				for (m=0; m<AXIS; ++m){
					if (compartment[nc].pTrans[m]!=nullv){
						value = compartment[nc].pTrans[m][symbol[o_v[i]].order] / S[m];
						fwrite(&value, sizeof(double), 1, fp1);
					}
				}
			}else
			if( symbol[o_v[i]].status&SYM_EXTRA )
			{
				for (m=0; m<AXIS; ++m){
					if (compartment[nc].pExtra[m]!=nullv){
						value = compartment[nc].pExtra[m][symbol[o_v[i]].order] / S[m];
						fwrite(&value, sizeof(double), 1, fp1);
					}
				}
			}else
			if( symbol[o_v[i]].status&SYM_INTRA )
			{
				for (m=0; m<AXIS; ++m){
					if (compartment[nc].pIntra[m]!=nullv){
						value = compartment[nc].pIntra[m][symbol[o_v[i]].order] / S[m];
						fwrite(&value, sizeof(double), 1, fp1);
					}
				}
			}
		}
	}
	fclose( fp1 );

	// preparation for stimulation .................................................................
	for(i=0;i<n_stm;++i)
	{
		stimulation[i].ts0=stimulation[i].stt;
		stimulation[i].tb0=stimulation[i].stt;
		stimulation[i].burst=1;
	}
	for (nc=0; nc<numCpm; ++nc)
	{
		cpm = (COMPARTMENT*)&(compartment[nc]);
		reaction = cpm->reaction;
		if(reaction[0])	// <html>Source</html>
		{
			stm=&(stimulation[0]);
			cpm->pSol[1]=stm->mn;
		}
	}

	// preparation for diffusion constant ..........................................................
	// calculation *********************************************************************************
	instp_h=calstp/2;
	instp_6=calstp/6;

	e1=(long)((xstp+0.4*outstp-xstt)/outstp-1);
	e2=(long)(outstp/calstp);
	time(&t);
	printf("calculation started...\n");
	printf("file= %s 00000/%05ld %s\n",flo,e1+1,ctime(&t));
	for(n1=0;n1<=e1;n1++){
		xx=((double)n1)*outstp;
		for(n2=1;n2<=e2;n2++){

#if N_GLOBAL_SOLUBLE
			for (nc=0; nc<numCpm; ++nc)
			for (i=0; i<N_GLOBAL_SOLUBLE; ++i)
				soluble[nc][gsl[i]] = soluble[refsl[i]][gsl[i]];
#endif
#if N_GLOBAL_TRANS
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_GLOBAL_TRANS; ++i)
				trans[nc][gtr[i]] = trans[reftr[i]][gtr[i]];
#endif
#if N_GLOBAL_EXTRA
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_GLOBAL_EXTRA; ++i)
				extra[nc][gex[i]] = extra[refex[i]][gex[i]];
#endif
#if N_GLOBAL_INTRA
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_GLOBAL_INTRA; ++i)
				intra[nc][gin[i]] = intra[refin[i]][gin[i]];
#endif
#if N_DIFFUSION_SOLUBLE
			for (nc=0; nc<numCpm; ++nc)
			for (i=0; i<N_DIFFUSION_SOLUBLE; ++i)
				lastsol[nc][i] = soluble[nc][dsl[i]];
#endif
#if N_DIFFUSION_TRANS
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_DIFFUSION_TRANS; ++i)
				lasttrans[nc][i] = trans[nc][dtr[i]];
#endif
#if N_DIFFUSION_EXTRA
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_DIFFUSION_EXTRA; ++i)
				lastextra[nc][i] = extra[nc][dex[i]];
#endif
#if N_DIFFUSION_INTRA
			for (nc=0; nc<numMem; ++nc)
			for (i=0; i<N_DIFFUSION_INTRA; ++i)
				lastintra[nc][i] = intra[nc][din[i]];
#endif
#ifdef CIRCUIT
			for (nc=0; nc<numCpm; ++nc)
				pastVm[nc] = soluble[nc][VM];
#endif


			for (nc=0; nc<numCpm; ++nc)
			{
				//if(compartment[nc]==NULL)
				//	continue;

				cpm = (COMPARTMENT*)&(compartment[nc]);
				reaction = cpm->reaction;
				tt=((double)n2)*calstp+xx;
#if N_SOLUBLE
				sl = cpm->pSol;
#endif
#if N_TRANS
				for(m=0;m<AXIS;++m)	if(cpm->pTrans[m]!=nullv) tr[m]=cpm->pTrans[m]; else tr[m] = nullv;
#endif
#if N_EXTRA
				for(m=0;m<AXIS;++m) if(cpm->pExtra[m]!=nullv) ex[m]=cpm->pExtra[m]; else ex[m] = nullv;
#endif
#if N_INTRA
				for(m=0;m<AXIS;++m) if(cpm->pIntra[m]!=nullv) in[m]=cpm->pIntra[m]; else in[m] = nullv;
#endif

				// stimulation .....................................................................
				if(reaction[0]) {	// <html>Source</html>
					stm=&(stimulation[0]);
					if((tt>=stm->stt) && (tt<=stm->stp)) {
						stm->ts1=tt-stm->ts0; stm->tb1=tt-stm->tb0;
						if(stm->BL==0) {stm->tb1=0;}
						if(( stm->burst == 1) && (stm->tb1 <= stm->BL)) {
							// step stimulation
							sl[1]=stm->mx;
						}else{
							if(stm->burst==1) {stm->tb0=tt; stm->burst=0; sl[1]=stm->mn;}
							tbi=tt-stm->tb0;
							if((stm->burst==0) && (tbi>stm->BI)) {stm->burst=1; stm->tb0=tt;}
						}
					}else{
						sl[1]=stm->mn;
					}
				}

				// pre-loop calculation.............................................................
				// integration .....................................................................
#if N_VARIABLE_SOLUBLE
				for(i=0; i<4; ++i)
					Memset_double( &(ksl[i][0]) ,0, N_VARIABLE_SOLUBLE);
#endif
#if N_VARIABLE_TRANS
				for(i=0; i<4; ++i)
				for(m=0; m<AXIS; ++m)
					if(cpm->pTrans[m]!=nullv)
					Memset_double( &(ktr[i][m][0]) ,0, N_VARIABLE_TRANS);
#endif
#if N_VARIABLE_EXTRA
				for(i=0; i<4; ++i)
				for(m=0; m<AXIS; ++m)
					if(cpm->pExtra[m]!=nullv)
					for(n=0; n<N_VARIABLE_EXTRA; ++n)
						kex[i][m][n]=0;
#endif
#if N_VARIABLE_INTRA
				for(i=0; i<4; ++i)
				for(m=0; m<AXIS; ++m)
					if(cpm->pIntra[m]!=nullv)
					for(n=0; n<N_VARIABLE_INTRA; ++n)
						kin[i][m][n]=0;
#endif
				nn=1;
				for( i=0; i<4; i++ )
				{
					if( i == 0 )
					{
#if N_VARIABLE_SOLUBLE
						for(n=0; n<N_VARIABLE_SOLUBLE; ++n)
							csl[n]=sl[vsl[n]];
#endif
#if N_VARIABLE_TRANS
						for(m=0; m<AXIS; ++m)
						if(cpm->pTrans[m]!=nullv)
						for(n=0; n<N_VARIABLE_TRANS; ++n)
							ctr[m][n]=tr[m][vtr[n]];
#endif
#if N_VARIABLE_EXTRA
						for(m=0; m<AXIS; ++m)
						if(cpm->pExtra[m]!=nullv)
						for(n=0; n<N_VARIABLE_EXTRA; ++n)
							cex[m][n]=ex[m][vex[n]];
#endif
#if N_VARIABLE_INTRA
						for(m=0; m<AXIS; ++m)
						if(cpm->pIntra[m]!=nullv)
						for(n=0; n<N_VARIABLE_INTRA; ++n)
							cin[m][n]=in[m][vin[n]];
#endif
					}
					else
					{
						if(i==1)
							nm=1;
						if(i==2)
							nm=0;
						if(i==3){
							nm=2;
							nn=2;
						}
						tt+=instp_h*((double)nm);
#if N_VARIABLE_SOLUBLE
						for(n4=0; n4<N_VARIABLE_SOLUBLE; ++n4)
							sl[vsl[n4]]=csl[n4]+ksl[i-1][n4]*instp_h*(double)nn;
#endif
#if N_VARIABLE_TRANS
						for(m=0; m<AXIS; ++m)
						if(cpm->pTrans[m]!=nullv)
						for(n4=0; n4<N_VARIABLE_TRANS; ++n4)
							tr[m][vtr[n4]]=ctr[m][n4]+ktr[i-1][m][n4]*instp_h*(double)nn;
#endif
#if N_VARIABLE_EXTRA
						for(m=0; m<AXIS; ++m)
						if(cpm->pExtra[m]!=nullv)
						for(n4=0; n4<N_VARIABLE_EXTRA; ++n4)
							ex[m][vex[n4]]=cex[m][n4]+kex[i-1][m][n4]*instp_h*(double)nn;
#endif
#if N_VARIABLE_INTRA
						for(m=0; m<AXIS; ++m)
						if(cpm->pIntra[m]!=nullv)
						for(n4=0; n4<N_VARIABLE_INTRA; ++n4)
							in[m][vin[n4]]=cin[m][n4]+kin[i-1][m][n4]*instp_h*(double)nn;
#endif
					}
					if(reaction[1]) {	// Ca^2^+ buffering
						ksl[i][0]+=-sl[4]*sl[1]*sl[0]+sl[3]*sl[2];
						ksl[i][2]+=sl[4]*sl[1]*sl[0]-sl[3]*sl[2];
					}
					if(reaction[1]) {
						for(j=0;j<AXIS;++j) if(cpm->pLastSol[j]) {
							if(cpm->organella_index == cpm->cpmAdj[j]->organella_index) if(cpm->cpmAdj[j]->reaction[1]) ksl[i][1]+=CDS[0][j]*(cpm->pLastSol[j][0]-sl[1]);
							else if(cpm->cpmAdj[j]->reaction[1]) ksl[i][1]+=cpm->area_density*CDS[0][j]*(cpm->pLastSol[j][0]-sl[1]);
						}
					}
				}
#if N_VARIABLE_SOLUBLE
				for(n5=0; n5<N_VARIABLE_SOLUBLE; ++n5)
					sl[vsl[n5]]=csl[n5]+instp_6*(ksl[0][n5]+2.0*(ksl[1][n5]+ksl[2][n5])+ksl[3][n5]);
#endif
#if N_VARIABLE_TRANS
				for(m=0; m<AXIS; ++m)
				if(cpm->pTrans[m]!=nullv)
				for(n5=0; n5<N_VARIABLE_TRANS; ++n5)
					tr[m][vtr[n5]]=ctr[m][n5]+instp_6*(ktr[0][m][n5]+2.0*(ktr[1][m][n5]+ktr[2][m][n5])+ktr[3][m][n5]);
#endif
#if N_VARIABLE_EXTRA
				for(m=0; m<AXIS; ++m)
				if(cpm->pExtra[m]!=nullv)
				for(n5=0; n5<N_VARIABLE_EXTRA; ++n5)
					ex[m][vex[n5]]=cex[m][n5]+instp_6*(kex[0][m][n5]+2.0*(kex[1][m][n5]+kex[2][m][n5])+kex[3][m][n5]);
#endif
#if N_VARIABLE_INTRA
				for(m=0; m<AXIS; ++m)
				if(cpm->pIntra[m]!=nullv)
				for(n5=0; n5<N_VARIABLE_INTRA; ++n5)
					in[m][vin[n5]]=cin[m][n5]+instp_6*(kin[0][m][n5]+2.0*(kin[1][m][n5]+kin[2][m][n5])+kin[3][m][n5]);
#endif
				// post-loop calculation............................................................
			}
		}
		// save values .............................................................................
		if ( (fp1=fopen(flo,"ab")) == NULL ) { printf("open error\n"); exit(0); }
		fwrite(&transform, sizeof(int), 1, fp1);
		for (i=0; i<n_outval; ++i)
		{
			for (nc=0; nc<numCpm; ++nc)
			{
				//if(compartment[j]==NULL)
				//	continue;
				if ((symbol[o_v[i]].status&SYM_CIRCUIT) && (symbol[o_v[i]].status&SYM_MEMBRANE))
				{
					if (compartment[nc].MA != 0)
						value = compartment[nc].pSol[symbol[o_v[i]].order] / compartment[nc].MA;
					else
						value = 0;
					for( m=0;m<AXIS;m++ )
					{
						if( compartment[nc].snmem[m] >= 0 )
							fwrite(&value, sizeof(double), 1, fp1);
					}
				}else
				if ((symbol[o_v[i]].status&SYM_CIRCUIT) || !(symbol[o_v[i]].status&SYM_MEMBRANE))
				{
					fwrite(&(compartment[nc].pSol[symbol[o_v[i]].order]), sizeof(double), 1, fp1);
				}else
				if( symbol[o_v[i]].status&SYM_TRANS )
				{
					for (m=0; m<AXIS; ++m){
						if (compartment[nc].pTrans[m]!=nullv){
							value = compartment[nc].pTrans[m][symbol[o_v[i]].order] / S[m];
							fwrite(&value, sizeof(double), 1, fp1);
						}
					}
				}else
				if( symbol[o_v[i]].status&SYM_EXTRA )
				{
					for (m=0; m<AXIS; ++m){
						if (compartment[nc].pExtra[m]!=nullv){
							value = compartment[nc].pExtra[m][symbol[o_v[i]].order] / S[m];
							fwrite(&value, sizeof(double), 1, fp1);
						}
					}
				}else
				if( symbol[o_v[i]].status&SYM_INTRA )
				{
					for (m=0; m<AXIS; ++m){
						if (compartment[nc].pIntra[m]!=nullv){
							value = compartment[nc].pIntra[m][symbol[o_v[i]].order] / S[m];
							fwrite(&value, sizeof(double), 1, fp1);
						}
					}
				}
			}
		}
		fclose( fp1 );

		time(&t);
		printf("file= %s %05ld/%05ld %s\n",flo,n1+1,e1+1,ctime(&t));
    }
//	for (i=0; i<n_sym; ++i)
//	{
//		free(symbol[i].name);
//		free(symbol[i].unit);
//	}

//*/
	return 0;
}

int InitSolubleSymbolArray(long* isl)
{
	isl[0]=0;	//B
	isl[1]=1;	//Ca^2^+
	isl[2]=2;	//Ca^2^+B
	isl[3]=3;	//k_b
	isl[4]=4;	//k_f
	return 1;
}
int InitTransMembraneSymbolArray(long* itr)
{
	return 1;
}
int InitExtraMembraneSymbolArray(long* iex)
{
	return 1;
}
int InitIntraMembraneSymbolArray(long* iin)
{
	return 1;
}



int InitVariableSolubleSymbolArray(long* vsl)
{
	vsl[0]=0;	//B
	vsl[1]=1;	//Ca^2^+
	vsl[2]=2;	//Ca^2^+B
	return 1;
}
int InitVariableTransMembraneSymbolArray(long* vtr)
{
	return 1;
}
int InitVariableExtraMembraneSymbolArray(long* vex)
{
	return 1;
}
int InitVariableIntraMembraneSymbolArray(long* vin)
{
	return 1;
}

int InitDiffusibleSolubleSymbolArray(long* dsl)
{
	dsl[0]=1;	//Ca^2^+
	return 1;
}
int InitDiffusibleTransMembraneSymbolArray(long* dtr)
{
	return 1;
}
int InitDiffusibleExtraMembraneSymbolArray(long* dex)
{
	return 1;
}
int InitDiffusibleIntraMembraneSymbolArray(long* din)
{
	return 1;
}

int InitGlobalSolubleSymbolArray(long* refsl, long* gsl)
{
	return 1;
}
int InitGlobalTransMembraneSymbolArray(long* reftr, long* gtr)
{
	return 1;
}
int InitGlobalExtraMembraneSymbolArray(long* refex, long* gex)
{
	return 1;
}
int InitGlobalIntraMembraneSymbolArray(long* refin, long* gin)
{
	return 1;
}

/*
//initialize array of long
void Memset_long(long *tgtAry,
				int setprm,
				unsigned int setsize)
{
	memset(tgtAry, setprm, setsize*sizeof(long));
}

//initialize array of double
void Memset_double(double *tgtAry,
				int setprm,
				unsigned int setsize)
{
	memset(tgtAry, setprm, setsize*sizeof(double));
}
*/

//initialize array of long
void Memset_long(long *tgtAry,
				long setprm,
				unsigned int setsize)
{
	int i;
	for(i=0; i<setsize; i++)
		tgtAry[i]=setprm;
}

//initialize array of double
void Memset_double(double *tgtAry,
				double setprm,
				unsigned int setsize)
{
	//memset(tgtAry, setprm, setsize*sizeof(double));
	int i;
	for(i=0; i<setsize; i++)
		tgtAry[i]=setprm;
}




